@@ -1,7 +1,12 @@
 """This module implements [Json Web Key RFC7517](https://tools.ietf.org/html/rfc7517)."""
 
-from .alg import ExpectedAlgRequired, UnsupportedAlg
-from .base import InvalidJwk, Jwk
+from .alg import (
+    ExpectedAlgRequired,
+    UnsupportedAlg,
+    select_alg_class,
+    select_alg_classes,
+)
+from .base import InvalidJwk, Jwk, UnsupportedKeyType, to_jwk
 from .ec import ECJwk, UnsupportedEllipticCurve
 from .jwks import JwkSet
 from .oct import SymmetricJwk
@@ -19,5 +24,9 @@ __all__ = [
     "SymmetricJwk",
     "UnsupportedAlg",
     "UnsupportedEllipticCurve",
+    "UnsupportedKeyType",
     "UnsupportedOKPCurve",
+    "select_alg_class",
+    "select_alg_classes",
+    "to_jwk",
 ]
